package WebService::Deltacloud::Types;
use strict;
use warnings;

use Moose;
use Moose::Util::TypeConstraints;
use MooseX::Types::Moose qw( Str HashRef );
use MooseX::Types -declare => [ qw(
    APITarget
    AWSProfile
    HTTPMethod
    UserAgent
    XMLSimple
) ];

use Net::Amazon::Config;
use LWP::UserAgent;
use XML::Simple qw( :strict );

# put the types in their own ghetto

# AWS credential profile from Net::Amazon::Config
class_type AWSProfile, { class => 'Net::Amazon::Config::Profile' };

coerce AWSProfile,
    from HashRef,
        via { Net::Amazon::Config->new( %{$_} )->get_profile };

# XML::Simple
class_type XMLSimple,
    { class => 'XML::Simple' };

coerce XMLSimple, 
    from HashRef,
        via { XML::Simple->new( %{$_} ) };

# LWP::UserAgent
class_type UserAgent,
    { class => 'LWP::UserAgent' };

coerce UserAgent,
    from HashRef,
        via { LWP::UserAgent->new( %{$_} ) };

# HTTP::Request methods
subtype HTTPMethod,
    as Str,
    where { $_ =~ /^(?:GET|PUT|POST|HEAD)$/ },
    message { "Method must be 'GET', 'PUT', 'POST', or 'HEAD'." };

# API targets
subtype APITarget,
    as Str,
    where { $_ =~ /^\//; },
    message { "API targets must start with '/'." };

no Moose;
1;
